'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _require = require('lodash'),
    assign = _require.assign,
    flatMapDeep = _require.flatMapDeep,
    isObject = _require.isObject,
    map = _require.map;

var _require2 = require('./constants'),
    TSIL_KEY = _require2.TSIL_KEY,
    JOIN_PATH_KEY = _require2.JOIN_PATH_KEY,
    ROOT_KEY = _require2.ROOT_KEY;

/**
 * Create a dot notation path with JOIN_PATH_KEY
 * @param  {String} from
 * @param  {String} to
 * @return {String} path
 */


var buildPath = function buildPath(from, to) {
  return from + JOIN_PATH_KEY + to;
};

/**
 * Iterate over all branch children.
 * If it's and object/array progress with recursivity
 * calling `flatten` method.
 * If not return an empty array which will be omited.
 *
 * @param  {Object} branch
 * @param  {String} path
 * @param  {Number} depth
 * @return {Any[]}
 */
var flattenNested = function flattenNested(branch, path, depth) {
  return map(branch, function (child, name) {
    return isObject(child) ? flatten(child, buildPath(path, name), depth + 1) : [];
  });
};

/**
 * Flat map to any level all the children into
 * single-level multiple array nodes.
 * Uses a recursivity strategy alongside with `flattenNested`.
 *
 * @param  {Object} subtree     - From initial one to any level
 * @param  {String} parentPath
 * @param  {Number} depth
 * @return {Object[]} list      - Tsil flatten list
 */
var flatten = function flatten(subtree, parentPath, depth) {
  return flatMapDeep(subtree, function (branch, name) {
    var path = buildPath(parentPath, name);
    var node = assign({}, branch, _defineProperty({}, TSIL_KEY, { path: path, parentPath: parentPath, depth: depth }));

    return [node, flattenNested(branch, path, depth)];
  });
};

/**
 * Starting point of flatten list.
 *
 * @param  {Object} tree
 * @return {Object[]} list - Tsil flatten list
 */
module.exports = function (tree) {
  return flatten(tree, ROOT_KEY, 0);
};